<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Version details
 *
 * @package    block_iclicker
 * @copyright  2012 i>clicker (R) <http://www.iclicker.com/dnn/> onwards Saji Nair (http://sajinair.com)
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

/**
 * Handles rendering the form for creating new pages and the submission of the form as well
 */

require_once('../../config.php');
global $CFG, $USER, $COURSE, $OUTPUT, $PAGE;
require_once('iclicker_service.php');
require_once('controller.php');

$site = get_site();
require_login($site);

// Activate the controller.
$cntlr = new iclicker_controller();
$cntlr->process_registration();
foreach ($cntlr->results as $key => $val) {
    $$key = $val;
}

// Begin rendering.
$PAGE->set_title( strip_tags($site->fullname).':'.iclicker_service::msg('app.iclicker').':'.iclicker_service::msg('reg.title') );
$PAGE->set_heading( iclicker_service::msg('app.iclicker').' '.iclicker_service::msg('reg.title') );
$PAGE->navbar->add(iclicker_service::msg('reg.title'));
$PAGE->set_focuscontrol('');
$PAGE->set_cacheable(false);
// NOTE: switching over to locally hosted JS and CSS files.
$PAGE->requires->js(iclicker_service::BLOCK_PATH.'/js/jquery-1.11.0.min.js', true);
$PAGE->requires->js(iclicker_service::BLOCK_PATH.'/js/jquery-ui-1.10.4.custom.min.js', true);
$PAGE->requires->css(iclicker_service::BLOCK_PATH.'/css/jquery-ui-1.10.4.custom.min.css');
$PAGE->requires->css(iclicker_service::BLOCK_PATH.'/css/iclicker.css');
$PAGE->set_url(iclicker_service::BLOCK_PATH.'/registration.php');
echo $OUTPUT->header();
?>
<div class="iclicker">

    <?php
    // Show messages if there are any to show.
    require('user_messages.php');
    ?>

    <div class="main_content">
        <div class="columns_container">
            <div class="left_column">
                <p><?php echo iclicker_service::msg('reg.remote.instructions') ?></p>

                <form method="post" id="registerForm" style="display: inline;">
                    <input type="hidden" name="register" value="true" />
                    <p class="highlighted">
                        <strong><?php echo iclicker_service::msg('reg.remote.id.enter') ?>:</strong>
                        <input name="clickerId" type="text" size="12" maxlength="12"
                               value="<?php echo $clickeridval ?>"/>
                        <input type="submit" class="registerButton" value="<?php echo iclicker_service::msg('app.register') ?>"
                               alt="<?php echo iclicker_service::msg('reg.register.submit.alt') ?>" />
                    </p>
                </form>

                <?php if (!empty($regs)) { ?>
                <table class="remotes" summary="<?php echo iclicker_service::msg('reg.registration.table.summary') ?>">
                    <colgroup>
                        <col width="40%" />
                        <col width="40%" />
                        <col />
                    </colgroup>
                    <tr>
                        <th><?php echo iclicker_service::msg('reg.remote.id.header') ?></th>
                        <th><?php echo iclicker_service::msg('reg.registered.date.header') ?></th>
                        <th>&nbsp;</th>
                    </tr>
                    <?php foreach ($regs as $reg) { ?>
                    <tr>
                        <td><?php echo $reg->clickerid ?></td>
                        <td><?php echo iclicker_service::df($reg->timecreated) ?></td>
                        <td>
                            <form method="post" style="display: inline;">
                                <input type="hidden" name="remove" value="remove" />
                                <input type="hidden" name="registrationId" value="<?php echo $reg->id ?>" />
                                <input type="submit" class="small" value="<?php
                                    echo iclicker_service::msg('app.remove');
                                ?>" alt="<?php
                                    echo iclicker_service::msg('reg.remove.submit.alt');
                                ?>" />
                            </form>
                        </td>
                    </tr>
                    <?php } ?>
                </table>
                <?php } ?>

                <?php if ($belowmessages) { ?>
                <!-- registration below messages area -->
                <div class="registration_below_messages_holder style5" style="margin-top: 1em;">
                    <div class="registration_below_messages">
                        <?php foreach ($belowmessages as $message) { ?>
                        <p class="registration_below_message"><?php echo $message ?></p>
                        <?php } ?>
                    </div>
                </div>
                <?php } ?>
            </div>

            <div class="right_column">
                <h2><?php echo iclicker_service::msg('reg.remote.faqs') ?></h2>

                <div id="accordion">
                    <h3><a href="#"><?php echo iclicker_service::msg('reg.remote.faq1.question') ?></a></h3>
                    <div>
                        <?php echo iclicker_service::msg('reg.remote.faq1.answer') ?>
                        <img src="img/clickers.png"
                             alt="<?php echo iclicker_service::msg('reg.remote.faq1.image.alt') ?>"/>
                    </div>

                    <h3><a href="#"><?php echo iclicker_service::msg('reg.remote.faq2.question') ?></a></h3>
                    <div><?php echo iclicker_service::msg('reg.remote.faq2.answer') ?></div>

                    <h3><a href="#"><?php echo iclicker_service::msg('reg.remote.faq3.question') ?></a></h3>
                    <div><?php echo iclicker_service::msg('reg.remote.faq3.answer') ?></div>

                    <h3><a href="#"><?php echo iclicker_service::msg('reg.remote.faq4.question') ?></a></h3>
                    <div><?php echo iclicker_service::msg('reg.remote.faq4.answer') ?></div>

                    <h3><a href="#"><?php echo iclicker_service::msg('reg.remote.faq5.question') ?></a></h3>
                    <div><?php echo iclicker_service::msg('reg.remote.faq5.answer') ?></div>

                    <h3><a href="#"><?php echo iclicker_service::msg('reg.remote.faq6.question') ?></a></h3>
                    <div><?php echo iclicker_service::msg('reg.remote.faq6.answer') ?></div>

                    <h3><a href="#"><?php echo iclicker_service::msg('reg.remote.faq7.question') ?></a></h3>
                    <div><?php echo iclicker_service::msg('reg.remote.faq7.answer') ?></div>

                    <h3><a href="#"><?php echo iclicker_service::msg('reg.remote.faq8.question') ?></a></h3>
                    <div><?php echo iclicker_service::msg('reg.remote.faq8.answer') ?></div>

                    <h3><a href="#"><?php echo iclicker_service::msg('reg.remote.faq9.question') ?></a></h3>
                    <div><?php echo iclicker_service::msg('reg.remote.faq9.answer') ?></div>

                </div>

            </div>
        </div>
    </div>

    <div class="nav_links">
        <?php
        $reglink = '<a class="nav_link current_nav_link" href="'.iclicker_service::block_url('registration.php').'">';
        $reglink .= iclicker_service::msg('reg.title').'</a>';
        $navlinks = $reglink.PHP_EOL;
        // The other links.
        if (iclicker_service::is_admin()) {
            $navlinks .= ' | <a class="nav_link" href="'.iclicker_service::block_url('admin.php').'">';
            $navlinks .= iclicker_service::msg('admin.title').'</a>'.PHP_EOL;
        } else if (iclicker_service::is_instructor()) {
            $navlinks .= ' | <a class="nav_link" href="'.iclicker_service::block_url('instructor.php').'">';
            $navlinks .= iclicker_service::msg('inst.title').'</a>'.PHP_EOL;
            if (iclicker_service::$blockiclickerssoenabled) {
                $navlinks .= ' | <a class="nav_link" href="'.iclicker_service::block_url('instructor_sso.php').'">';
                $navlinks .= iclicker_service::msg('inst.sso.title').'</a>'.PHP_EOL;
            }
        }
        echo $navlinks;
        ?>
    </div>

    <div class="iclicker_version">Version <?php
            echo iclicker_service::VERSION;
        ?> (<?php
            echo iclicker_service::BLOCK_VERSION;
            ?>)</div>

</div>

<script type="text/javascript">
    $(document).ready(function() {
        $("#accordion").accordion({ active: 0,
            alwaysOpen: false,
            animated: false,
            autoHeight: false,
            collapsible: true,
            heightStyle: "content"
        });
    });
</script>

<?php
    echo $OUTPUT->footer();
