<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Version details
 *
 * @package    block_iclicker
 * @copyright  2012 i>clicker (R) <http://www.iclicker.com/dnn/> onwards Saji Nair (http://sajinair.com)
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

defined('MOODLE_INTERNAL') || die;

// Control the config settings for this plugin.
require_once('iclicker_service.php');
$blockname = iclicker_service::BLOCK_NAME;
if ($ADMIN->fulltree) {
    // General.
    $settings->add(
            new admin_setting_heading('block_iclicker_general_heading',
                    get_string('config_general', $blockname),
                    null
            )
    );
    $settings->add(
            new admin_setting_configtext(iclicker_service::BLOCK_NAME . '/block_iclicker_notify_emails',
                    get_string('config_notify_emails', $blockname),
                    get_string('config_notify_emails_desc', $blockname),
                    '',
                    PARAM_TEXT,
                    50
            )
    );
    /*
    $settings->add(
        new admin_setting_configcheckbox(iclicker_service::BLOCK_NAME.'/block_iclicker_allow_sharing',
            get_string('config_allow_sharing', $blockname),
            get_string('config_allow_sharing_desc', $blockname),
            0
        )
    );
    */
    $settings->add(
            new admin_setting_configcheckbox(iclicker_service::BLOCK_NAME . '/block_iclicker_enable_shortname',
                    get_string('config_enable_shortname', $blockname),
                    get_string('config_enable_shortname_desc', $blockname),
                    0
            )
    );
    $settings->add(
            new admin_setting_configcheckbox(iclicker_service::BLOCK_NAME . '/block_iclicker_disable_alternateid',
                    get_string('config_disable_alternateid', $blockname),
                    get_string('config_disable_alternateid_desc', $blockname),
                    0
            )
    );
    $settings->add(
            new admin_setting_configtext(iclicker_service::BLOCK_NAME . '/block_iclicker_max_courses_fetched',
                    get_string('config_max_courses', $blockname),
                    get_string('config_max_courses_desc', $blockname),
                    '100',
                    PARAM_INT,
                    5
            )
    );
    // WS
    /*
     * webservices code is currently disabled ********************
     * $settings->add(
     * new admin_setting_heading('block_iclicker_ws_heading',
     * get_string('config_webservices', $blockname),
     * null
     * )
     * );
     * $settings->add(
     * new admin_setting_configcheckbox('block_iclicker_use_national_ws',
     * get_string('config_use_national_ws', $blockname),
     * get_string('config_use_national_ws_desc', $blockname),
     * 0
     * )
     * );
     * $settings->add(
     * new admin_setting_configtext('block_iclicker_domain_url',
     * get_string('config_domain_url', $blockname),
     * get_string('config_domain_url_desc', $blockname),
     * '', //50,200
     * PARAM_TEXT,
     * 50
     * )
     * );
     * $settings->add(
     * new admin_setting_configtext('block_iclicker_webservices_url',
     * get_string('config_webservices_url', $blockname),
     * get_string('config_webservices_url_desc', $blockname),
     * '', //iclicker_service::NATIONAL_WS_URL, //50,200
     * PARAM_TEXT,
     * 75
     * )
     * );
     * $settings->add(
     * new admin_setting_configtext('block_iclicker_webservices_username',
     * get_string('config_webservices_username', $blockname),
     * get_string('config_webservices_username_desc', $blockname),
     * '', //iclicker_service::NATIONAL_WS_URL, //50,200
     * PARAM_TEXT,
     * 30
     * )
     * );
     * $settings->add(
     * new admin_setting_configtext('block_iclicker_webservices_password',
     * get_string('config_webservices_password', $blockname),
     * get_string('config_webservices_password_desc', $blockname),
     * '', //iclicker_service::NATIONAL_WS_URL, //50,200
     * PARAM_TEXT,
     * 30
     * )
     * );
     */
    // SSO.
    $headerdesc = get_string('config_sso_disabled', $blockname);
    $currentssokey = get_config($blockname, 'block_iclicker_sso_shared_key');
    if (!empty($currentssokey)) {
        $headerdesc = get_string('config_sso_enabled', $blockname);
        $timestamp = time();
        $headerdesc .= ' [Sample encoded key: ' . iclicker_service::make_encoded_key($timestamp) . '|' . $timestamp . ']';
    }
    $settings->add(
            new admin_setting_heading('block_iclicker_sso_heading',
                    get_string('config_sso', $blockname),
                    $headerdesc
            )
    );
    $settings->add(
            new admin_setting_configtext(iclicker_service::BLOCK_NAME . '/block_iclicker_sso_shared_key',
                    get_string('config_shared_key', $blockname),
                    get_string('config_shared_key_desc', $blockname),
                    '',
                    PARAM_TEXT,
                    50
            )
    );
}

