<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Version details
 *
 * @package    block_iclicker
 * @copyright  2012 i>clicker (R) <http://www.iclicker.com/dnn/> onwards Saji Nair (http://sajinair.com)
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
/* $Id$ */

if (!defined('MOODLE_INTERNAL')) {
    die('Direct access to this script is forbidden.'); // It must be included from a Moodle page
}

require_once(dirname(__FILE__).'/../../../config.php');
global $CFG, $USER, $COURSE;
// Link in external libraries.
require_once($CFG->dirroot.'/blocks/iclicker/iclicker_service.php');

/**
 * Unit tests for the iclicker services
 * Execute tests at:
 * moodle/admin/report/unittest/index.php?path=blocks%2Ficlicker
 */
class iclicker_services_test extends UnitTestCase {

    public $courseid = 1;
    public $clickerid = '99996666';

    public $studentid1 = 1;
    public $studentid2 = 2;

    public $catname = 'az_category';
    public $itemname = 'az_gradeitem';
    public $gradescore = 91;

    public function cleanup() {
        // Cleanup any clickers before the test.
        $userid = iclicker_service::require_user();
        $results = iclicker_service::get_registrations_by_user($userid);
        if ($results) {
            echo "cleanup registrations for user: $userid  ".PHP_EOL;
            foreach ($results as $reg) {
                if ($reg->clickerid == $this->clickerid) {
                    iclicker_service::remove_registration($reg->id);
                    echo "cleanup: $reg->id ".PHP_EOL;
                }
            }
        }
        // Cleanup the test grades.
        $defgradecats = grade_category::fetch_all( array(
            'courseid' => $this->courseid,
            'fullname' => iclicker_service::GRADE_CATEGORY_NAME
            )
        );
        $stuffgradecats = grade_category::fetch_all( array(
            'courseid' => $this->courseid,
            'fullname' => 'stuff'
            )
        );
        $gradecats = $defgradecats;
        if (is_array($defgradecats) && is_array($stuffgradecats)) {
            $gradecats = array_merge($defgradecats, $stuffgradecats);
        } else if (is_array($stuffgradecats)) {
            $gradecats = $stuffgradecats;
        }
        if ($gradecats) {
            foreach ($gradecats as $cat) {
                $gradeitems = grade_item::fetch_all(array(
                    'courseid' => $this->courseid,
                    'categoryid' => $cat->id
                    )
                );
                if ($gradeitems) {
                    foreach ($gradeitems as $item) {
                        $grades = grade_grade::fetch_all(array(
                            'itemid' => $item->id
                            )
                        );
                        if ($grades) {
                            foreach ($grades as $grade) {
                                $grade->delete("cleanup");
                            }
                        }
                        $item->delete("cleanup");
                    }
                }
                $cat->delete("cleanup");
            }
        }
    }

    public function setUp() {
        $this->cleanup();
        iclicker_service::$testmode = true;
    }

    public function tearDown() {
        iclicker_service::$testmode = false;
        $this->cleanup();
    }

    public function test_arrays_subtract() {
        // Array_diff and array_diff_key are the same as a subtract when used with 2 arrays -- array_diff(A1, A2) => A1 - A2.
        $a1 = array(1, 2, 3, 4, 5);
        $a2 = array(3, 4, 5, 6, 7);

        $result = array_values(array_diff($a1, $a2));
        $this->assertEqual(2, count($result));
        $this->assertTrue(in_array(1, $result));
        $this->assertTrue(in_array(2, $result));
        $result = array_values(array_diff($a2, $a1));
        $this->assertEqual(2, count($result));
        $this->assertTrue(in_array(6, $result));
        $this->assertTrue(in_array(7, $result));

        $result = array_values(array_intersect($a1, $a2));
        $this->assertEqual(3, count($result));
        $this->assertTrue(in_array(3, $result));
        $this->assertTrue(in_array(4, $result));
        $this->assertTrue(in_array(5, $result));
        $result = array_values(array_intersect($a2, $a1));
        $this->assertEqual(3, count($result));
        $this->assertTrue(in_array(3, $result));
        $this->assertTrue(in_array(4, $result));
        $this->assertTrue(in_array(5, $result));

        $a1 = array('A' => 1, 'B' => 2, 'C' => 3, 'D' => 4, 'E' => 5);
        $a2 = array('C' => 3, 'D' => 4, 'E' => 5, 'F' => 6, 'G' => 7);

        $result = array_values(array_diff_key($a1, $a2));
        $this->assertEqual(2, count($result));
        $this->assertTrue(in_array(1, $result));
        $this->assertTrue(in_array(2, $result));
        $result = array_values(array_diff_key($a2, $a1));
        $this->assertEqual(2, count($result));
        $this->assertTrue(in_array(6, $result));
        $this->assertTrue(in_array(7, $result));

        $result = array_values(array_intersect_key($a1, $a2));
        $this->assertEqual(3, count($result));
        $this->assertTrue(in_array(3, $result));
        $this->assertTrue(in_array(4, $result));
        $this->assertTrue(in_array(5, $result));
        $result = array_values(array_intersect_key($a2, $a1));
        $this->assertEqual(3, count($result));
        $this->assertTrue(in_array(3, $result));
        $this->assertTrue(in_array(4, $result));
        $this->assertTrue(in_array(5, $result));
    }

    public function test_assert() {
        $this->assertEqual("AZ", "AZ");
        $this->assertEqual(iclicker_service::$testmode, true);
    }

    public function test_require_user() {
        $userid = iclicker_service::require_user();
        $this->assertTrue($userid);
    }

    public function test_get_users() {
        $userid = iclicker_service::require_user();
        $this->assertTrue($userid);
        $results = iclicker_service::get_users(array($userid));
        $this->assertTrue($results);
        $this->assertTrue(count($results) == 1);
        $this->assertEqual($results[$userid]->id, $userid);
    }

    public function test_validate_clickerid() {
        $clickerid = null;
        try {
            iclicker_service::validate_clicker_id($clickerid);
            $this->fail("should have died");
        } catch (ClickerIdInvalidException $e) {
            $this->assertEqual(ClickerIdInvalidException::F_EMPTY, $e->type);
        }

        $clickerid = "XXX";
        try {
            iclicker_service::validate_clicker_id($clickerid);
            $this->fail("should have died");
        } catch (ClickerIdInvalidException $e) {
            $this->assertEqual(ClickerIdInvalidException::F_CHARS, $e->type);
        }

        $clickerid = "00000000000";
        try {
            iclicker_service::validate_clicker_id($clickerid);
            $this->fail("should have died");
        } catch (ClickerIdInvalidException $e) {
            $this->assertEqual(ClickerIdInvalidException::F_LENGTH, $e->type);
        }

        $clickerid = iclicker_service::CLICKERID_SAMPLE;
        try {
            iclicker_service::validate_clicker_id($clickerid);
            $this->fail("should have died");
        } catch (ClickerIdInvalidException $e) {
            $this->assertEqual(ClickerIdInvalidException::F_SAMPLE, $e->type);
        }

        $clickerid = "ABCD0123";
        try {
            iclicker_service::validate_clicker_id($clickerid);
            $this->fail("should have died");
        } catch (ClickerIdInvalidException $e) {
            $this->assertEqual(ClickerIdInvalidException::F_CHECKSUM, $e->type);
        }

        $clickerid = "112233";
        $result = iclicker_service::validate_clicker_id($clickerid);
        $this->assertEqual($result, "00112233");

        $clickerid = "11111111";
        $result = iclicker_service::validate_clicker_id($clickerid);
        $this->assertEqual($result, $clickerid);
    }

    public function test_alternate_clickerid() {
        $result = null;

        $result = iclicker_service::translate_clicker_id(null);
        $this->assertEqual(null, $result);

        $result = iclicker_service::translate_clicker_id("");
        $this->assertEqual(null, $result);

        $result = iclicker_service::translate_clicker_id(iclicker_service::CLICKERID_SAMPLE);
        $this->assertEqual(null, $result);

        $result = iclicker_service::translate_clicker_id("11111111");
        $this->assertEqual(null, $result);

        $result = iclicker_service::translate_clicker_id("22222222");
        $this->assertEqual("02222202", $result);

        $result = iclicker_service::translate_clicker_id("33333333");
        $this->assertEqual(null, $result);

        $result = iclicker_service::translate_clicker_id("44444444");
        $this->assertEqual("04444404", $result);

        $result = iclicker_service::translate_clicker_id("55555555");
        $this->assertEqual(null, $result);

        $result = iclicker_service::translate_clicker_id("66666666");
        $this->assertEqual(null, $result);

        $result = iclicker_service::translate_clicker_id("77777777");
        $this->assertEqual(null, $result);

        $result = iclicker_service::translate_clicker_id("88888888");
        $this->assertEqual("08888808", $result);

        $result = iclicker_service::translate_clicker_id("99999999");
        $this->assertEqual(null, $result);

        $result = iclicker_service::translate_clicker_id("40404040");
        $this->assertEqual("00404000", $result);
    }

    public function test_make_clickerid_dates() {
        $userid = iclicker_service::require_user();
        $reg1 = new stdClass;
        $reg1->clickerid = '11111111';
        $reg1->ownerid = 'aaronz';
        $reg1->timecreated = time() - 1000;
        $reg2 = new stdClass;
        $reg2->clickerid = '22222222';
        $reg2->ownerid = 'beckyz';
        $reg2->timecreated = time() - 100;
        $result = null;

        $clickers = array($reg1);
        $result = iclicker_service::make_clicker_ids_and_dates($clickers);
        $this->assertNotNull($result);
        $this->assertEqual("11111111", $result['clickerid']);

        $clickers = array($reg2);
        $result = iclicker_service::make_clicker_ids_and_dates($clickers);
        $this->assertNotNull($result);
        $this->assertEqual("22222222,02222202", $result['clickerid']);

        $clickers[] = $reg1;
        $result = iclicker_service::make_clicker_ids_and_dates($clickers);
        $this->assertNotNull($result);
        $this->assertEqual("22222222,02222202,11111111", $result['clickerid']);
    }

    public function test_alphanumeric_gen() {
        $rand = iclicker_service::random_alpha_numeric(10);
        $this->assertNotNull($rand);
        $this->assertTrue(strlen($rand) == 10);

        $rand = iclicker_service::random_alpha_numeric(12);
        $this->assertNotNull($rand);
        $this->assertTrue(strlen($rand) == 12);
    }

    /**
     * Sample key:
     * abcdef1234566890
     * Sample timestamp:
     * 1332470760
     * Encoded key:
     * cc80462bfc0da7e614237d7cab4b7971b0e71e9f|1332470760
     */
    public function test_sso_key_encoding() {
        $key = "abcdef1234566890";
        iclicker_service::set_shared_key($key);

        // test expired timestamp
        $encodedkey = "cc80462bfc0da7e614237d7cab4b7971b0e71e9f|1332470760";
        try {
            iclicker_service::verify_key($encodedkey);
            $this->fail("should have died");
        } catch (ClickerSecurityException $e) {
            $this->assertNotNull($e->getMessage());
        }

        // test invalid format
        try {
            iclicker_service::verify_key("xxxxxxxxxxxxx");
            $this->fail("should have died");
        } catch (InvalidArgumentException $e) {
            $this->assertNotNull($e->getMessage());
        }
        try {
            iclicker_service::verify_key("xxxxxxxxxxxxx|");
            $this->fail("should have died");
        } catch (InvalidArgumentException $e) {
            $this->assertNotNull($e->getMessage());
        }
        try {
            iclicker_service::verify_key("xxxxxxxx|12344ffff");
            $this->fail("should have died");
        } catch (InvalidArgumentException $e) {
            $this->assertNotNull($e->getMessage());
        }

        // test valid encoded key
        $timestamp = time();
        $encodedkey = sha1($key . ":" . $timestamp) . '|' . $timestamp;
        $result = iclicker_service::verify_key($encodedkey);
        $this->assertTrue($result);
        echo "<div><b>SSO key:</b> key=$key, ts=$timestamp <br/> encoded=<input type='text' size='".(strlen($encodedkey) + 2)."' value='$encodedkey'/></div>".PHP_EOL;
    }

    public function test_user_keys() {
        global $DB;
        $userid = iclicker_service::require_user();
        $this->assertNotNull($userid);

        // saving key for user should work
        $userkey = iclicker_service::make_user_key($userid, true);
        $this->assertNotNull($userkey);

        $keyscount = $DB->count_records(iclicker_service::USER_KEY_TABLENAME, array('userid' => $userid));
        $this->assertEqual(1, $keyscount);

        // getting key for user (from make)
        $userkey1 = iclicker_service::make_user_key($userid, false);
        $this->assertNotNull($userkey1);
        $this->assertEqual($userkey, $userkey1);

        // getting key for user (from get)
        $userkey1 = iclicker_service::get_user_key($userid);
        $this->assertNotNull($userkey1);
        $this->assertEqual($userkey, $userkey1);

        // update the key should work
        $userkey2 = iclicker_service::make_user_key($userid, true);
        $this->assertNotNull($userkey2);
        $this->assertNotEqual($userkey, $userkey2);

        // trying to save another key for this user should fail
        $dupeuserkey = new stdClass();
        $dupeuserkey->userid = $userid;
        $dupeuserkey->userkey = 'x12xx3x3x43x44';
        try {
            $DB->insert_record(iclicker_service::USER_KEY_TABLENAME, $dupeuserkey);
            $this->fail("Should have thrown an exception");
        } catch (dml_exception $e) {
            $this->assertNotNull($e->getMessage());
        }

        // delete the key should work
        $DB->delete_records(iclicker_service::USER_KEY_TABLENAME, array('userid' => $userid));
        $userkey3 = iclicker_service::get_user_key($userid);
        $this->assertNull($userkey3);

        // adding key2 should work since key1 is gone
        $userkey4 = iclicker_service::make_user_key($userid, true);
        $this->assertNotNull($userkey4);

        // cleanup
        $DB->delete_records(iclicker_service::USER_KEY_TABLENAME, array('userid' => $userid));
    }

    public function test_registrations() {
        $reg = null;
        $userid = iclicker_service::require_user();

        // try get registration
        $reg = iclicker_service::get_registration_by_clicker_id($this->clickerid);
        $this->assertFalse($reg);

        // create registration
        $reg = iclicker_service::create_clicker_registration($this->clickerid, $userid);
        $this->assertTrue($reg);
        $this->assertEqual($this->clickerid, $reg->clickerid);
        $regid = $reg->id;
        $this->assertTrue($regid);
        $this->assertFalse($reg->fromnational);

        // get registration
        $reg1 = iclicker_service::get_registration_by_clicker_id($this->clickerid, $userid);
        $this->assertTrue($reg1);
        $this->assertEqual($this->clickerid, $reg1->clickerid);
        $this->assertEqual($regid, $reg1->id);

        $reg2 = iclicker_service::get_registration_by_id($regid);
        $this->assertTrue($reg2);
        $this->assertEqual($this->clickerid, $reg2->clickerid);
        $this->assertEqual($regid, $reg2->id);

        // save registration
        $reg->fromnational = 1;
        $saveid = iclicker_service::save_registration($reg);
        $this->assertTrue($saveid);
        $this->assertEqual($regid, $saveid);

        // check it changed
        $reg3 = iclicker_service::get_registration_by_id($regid);
        $this->assertTrue($reg3);
        $this->assertEqual($regid, $reg3->id);
        $this->assertTrue($reg3->fromnational);
        // too fast $this->assertNotEqual($reg->timemodified, $reg3->timemodified);

        // make registration inactive
        $this->assertTrue($reg->activated);
        $reg4 = iclicker_service::set_registration_active($regid, false);
        $this->assertTrue($reg4);
        $this->assertFalse($reg4->activated);
        // check it changed
        $reg5 = iclicker_service::get_registration_by_id($regid);
        $this->assertTrue($reg5);
        $this->assertEqual($regid, $reg5->id);
        $this->assertEqual($reg4->id, $reg5->id);
        $this->assertFalse($reg5->activated);

        // get all registration
        $results = iclicker_service::get_registrations_by_user($userid);
        $this->assertTrue($results);
        $this->assertEqual(1, count($results));

        $results = iclicker_service::get_registrations_by_user($userid, true);
        $this->assertNotNull($results);
        $this->assertFalse($results);
        $this->assertEqual(0, count($results));

        $results = iclicker_service::get_all_registrations();
        $this->assertTrue($results);
        $this->assertTrue(count($results) >= 1);

        // remove registration
        $result = iclicker_service::remove_registration($regid);
        $this->assertTrue($result);

        // try get registration
        $reg = iclicker_service::get_registration_by_id($regid);
        $this->assertFalse($reg);
    }

    public function test_encode_decode() {
        $xml = <<<XML
<Register>
  <S DisplayName="DisplayName-azeckoski-123456" FirstName="First" LastName="Lastazeckoski-123456"
    StudentID="101" Email="azeckoski-123456@email.com" URL="http://sakaiproject.org" ClickerID="11111111"></S>
</Register>
XML;
        $result = iclicker_service::decode_registration($xml);
        $this->assertNotNull($result);
        $this->assertNotNull($result->clickerid);
        $this->assertNotNull($result->ownerid);
        $this->assertEqual($result->clickerid, '11111111');
        $this->assertEqual($result->ownerid, 101);
        // $this->assertEqual($result->user_username, 'student01'); // DISABLED - username might be different for this user id

        $xml = <<<XML
<coursegradebook courseid="BFW61">
  <user id="101" usertype="S">
    <lineitem name="05/05/2009" pointspossible="100" type="iclicker polling scores" score="100"/>
    <lineitem name="06/06/2009" pointspossible="50" type="iclicker polling scores" score="50"/>
  </user>
  <user id="102" usertype="S">
    <lineitem name="06/06/2009" pointspossible="50" type="iclicker polling scores" score="30"/>
    <lineitem name="07/07/2009" pointspossible="100" type="iclicker polling scores" score="80"/>
  </user>
</coursegradebook>
XML;
        $result = iclicker_service::decode_gradebook($xml);
        $this->assertNotNull($result);
        $this->assertNotNull($result->course_id);
        $this->assertEqual($result->course_id, 'BFW61');
        $this->assertNotNull($result->students);
        $this->assertNotNull($result->items);
        $this->assertEqual(count($result->students), 2);
        $this->assertEqual(count($result->items), 3);
        $this->assertNotNull($result->items['05/05/2009']);
        $this->assertNotNull($result->items['06/06/2009']);
        $this->assertNotNull($result->items['07/07/2009']);

        $xml = <<<XML
<StudentRoster>
    <S StudentID="student01" FirstName="student01" LastName="student01" URL="https://www.iclicker.com/" CourseName="">
        <Registration ClickerId="11111111" WhenAdded="2009-01-27" Enabled="True" />
        <Registration ClickerId="22222222" WhenAdded="2009-01-27" Enabled="True" />
    </S>
</StudentRoster>
XML;
        $result = iclicker_service::decode_ws_xml($xml);
        $this->assertNotNull($result);
        $this->assertTrue(is_array($result));
        $this->assertEqual(count($result), 2);
        $this->assertNotNull($result[0]);
        $this->assertNotNull($result[0]->clickerid);
        $this->assertNotNull($result[0]->ownerid);
        $this->assertNotNull($result[0]->timecreated);
        $this->assertNotNull($result[0]->activated);
        $this->assertEqual($result[0]->clickerid, '11111111');
        $this->assertEqual($result[0]->user_username, 'student01');
        /* Fails in windows for some reason - not sure why
        Fail: blocks/iclicker/simpletest/test_iclicker_service.php / ► iclicker_services_test / ► test_encode_decode / ►
        Equal expectation fails because [Integer: 1233032400] differs from [Integer: 1233014400] by 18000
        */
        // $this->assertEqual($result[0]->timecreated, 1233014400);
        $this->assertEqual($result[0]->activated, true);

        // no good way to test this right now
        // $result = iclicker_service::encode_courses($instructorid);
        // $result = iclicker_service::encode_enrollments($courseid);
        // $result = iclicker_service::encode_gradebook_results($courseid, $result_items);

        $clickerregistration = new stdClass();
        $clickerregistration->ownerid = 101;
        $clickerregistration->clickerid = '12345678';
        $clickerregistration->activated = true;
        $result = iclicker_service::encode_registration($clickerregistration);
        $this->assertNotNull($result);
        // $this->assertTrue(stripos($result, 'student01') > 0); // DISABLED - username might be different for this user id
        $this->assertTrue(stripos($result, '12345678') > 0);
        $this->assertTrue(stripos($result, 'True') > 0);

        $registrations = array($clickerregistration);
        $result = iclicker_service::encode_registration_result($registrations, true, 'hello');
        $this->assertNotNull($result);
        $this->assertTrue(stripos($result, 'True') > 0);
        $this->assertTrue(stripos($result, 'hello') > 0);
        $result = iclicker_service::encode_registration_result($registrations, false, 'failed');
        $this->assertNotNull($result);
        $this->assertTrue(stripos($result, 'False') > 0);
        $this->assertTrue(stripos($result, 'failed') > 0);

    }

    public function test_save_grades() {
        $testitemname1 = 'testing-iclicker-item1';

        $testitemname2 = 'testing-iclicker-item2';
        $testitemname3 = 'testing-iclicker-item3';

        $gradebook = new stdClass();

        // saving a gradebook with no course_id not allowed
        try {
            $result = iclicker_service::save_gradebook($gradebook);
            $this->fail("should have died");
        } catch (Exception $e) {
            $this->assertNotNull($e);
        }

        $gradebook->course_id = $this->courseid;
        $gradebook->items = array();

        // saving an empty gradebook not allowed
        try {
            $result = iclicker_service::save_gradebook($gradebook);
            $this->fail("should have died");
        } catch (Exception $e) {
            $this->assertNotNull($e);
        }

        // saving one with one valid item
        $score = new stdClass();
        $score->userid = 1;
        $score->score = 75.0;

        $gradeitem = new stdClass();
        $gradeitem->name = $testitemname1;
        $gradeitem->points_possible = 90;
        $gradeitem->type = iclicker_service::GRADE_CATEGORY_NAME;
        $gradeitem->scores = array();

        $gradeitem->scores[] = $score;
        $gradebook->items[] = $gradeitem;

        $result = iclicker_service::save_gradebook($gradebook);
        $this->assertNotNull($result);
        $this->assertNotNull($result->course_id);
        $this->assertNotNull($result->course);
        $this->assertNotNull($result->default_category_id);
        $this->assertNotNull($result->items);
        $this->assertEqual($result->course_id, $this->courseid);
        $this->assertEqual(count($result->items), 1);
        $this->assertNotNull($result->items[0]);
        $this->assertNotNull($result->items[0]->id);
        $this->assertNotNull($result->items[0]->scores);
        $this->assertEqual(count($result->items[0]->scores), 1);
        $this->assertFalse(isset($result->items[0]->errors));
        $this->assertEqual($result->items[0]->grademax, 90);
        $this->assertEqual($result->items[0]->iteminfo, iclicker_service::GRADE_CATEGORY_NAME);
        $this->assertNotNull($result->items[0]->categoryid);
        $this->assertEqual($result->items[0]->courseid, $result->course_id);
        $this->assertEqual($result->items[0]->itemname, $testitemname1);
        $this->assertNotNull($result->items[0]->scores[0]);
        $this->assertFalse(isset($result->items[0]->scores[0]->error));

        // Saving one with multiple items, some invalid.
        $gradeitem->type = 'stuff'; // Update category.
        $score->score = 50; // SCORE_UPDATE_ERRORS.

        $score1 = new stdClass();
        $score1->userid = 'xxxxxx'; // USER_DOES_NOT_EXIST_ERROR.
        $score1->score = 80;
        $gradeitem->scores[] = $score1;

        $score2 = new stdClass();
        $score2->userid = '2';
        $score2->score = 101; // POINTS_POSSIBLE_UPDATE_ERRORS.
        $gradeitem->scores[] = $score2;

        $score3 = new stdClass();
        $score3->userid = '3';
        $score3->score = 'XX'; // GENERAL_ERRORS.
        $gradeitem->scores[] = $score3;

        $result = iclicker_service::save_gradebook($gradebook);
        $this->assertNotNull($result);
        $this->assertNotNull($result->course_id);
        $this->assertNotNull($result->course);
        // $this->assertNotNull($result->default_category_id);
        $this->assertNotNull($result->items);
        $this->assertEqual($result->course_id, $this->courseid);
        $this->assertEqual(count($result->items), 1);
        $this->assertNotNull($result->items[0]);
        $this->assertNotNull($result->items[0]->id);
        $this->assertEqual($result->items[0]->iteminfo, 'stuff');
        $this->assertNotNull($result->items[0]->scores);
        $this->assertEqual(count($result->items[0]->scores), 4);
        $this->assertTrue(isset($result->items[0]->errors));
        $this->assertEqual(count($result->items[0]->errors), 4);
        $this->assertEqual($result->items[0]->grademax, 90);
        $this->assertNotNull($result->items[0]->categoryid);
        $this->assertEqual($result->items[0]->courseid, $result->course_id);
        $this->assertEqual($result->items[0]->itemname, $testitemname1);
        $this->assertNotNull($result->items[0]->scores[0]);
        $this->assertTrue(isset($result->items[0]->scores[0]->error));
        $this->assertEqual($result->items[0]->scores[0]->error, iclicker_service::SCORE_UPDATE_ERRORS);
        $this->assertNotNull($result->items[0]->scores[1]);
        $this->assertTrue(isset($result->items[0]->scores[1]->error));
        $this->assertEqual($result->items[0]->scores[1]->error, iclicker_service::USER_DOES_NOT_EXIST_ERROR);
        $this->assertNotNull($result->items[0]->scores[2]);
        $this->assertTrue(isset($result->items[0]->scores[2]->error));
        $this->assertEqual($result->items[0]->scores[2]->error, iclicker_service::POINTS_POSSIBLE_UPDATE_ERRORS);
        $this->assertNotNull($result->items[0]->scores[3]);
        $this->assertTrue(isset($result->items[0]->scores[3]->error));
        $this->assertEqual($result->items[0]->scores[3]->error, 'SCORE_INVALID');

        $xml = iclicker_service::encode_gradebook_results($result);
        $this->assertNotNull($xml);
        $this->assertTrue(stripos($xml, '<user ') > 0);
        $this->assertTrue(stripos($xml, '<lineitem ') > 0);
        $this->assertTrue(stripos($xml, '<error ') > 0);
        $this->assertTrue(stripos($xml, iclicker_service::SCORE_UPDATE_ERRORS) > 0);
        $this->assertTrue(stripos($xml, iclicker_service::USER_DOES_NOT_EXIST_ERROR) > 0);
        $this->assertTrue(stripos($xml, iclicker_service::POINTS_POSSIBLE_UPDATE_ERRORS) > 0);
        $this->assertTrue(stripos($xml, iclicker_service::GENERAL_ERRORS) > 0);
        // echo "<xmp>$xml</xmp>";

        // Save 1 update and 2 new grades.
        $score->score = 85;
        $score2->score = 50;
        $score3->score = 0;
        $gradeitem->scores = array();
        $gradeitem->scores[] = $score;
        $gradeitem->scores[] = $score2;
        $gradeitem->scores[] = $score3;

        $result = iclicker_service::save_gradebook($gradebook);
        $this->assertNotNull($result);
        $this->assertNotNull($result->course_id);
        $this->assertNotNull($result->items);
        $this->assertEqual($result->course_id, $this->courseid);
        $this->assertEqual(count($result->items), 1);
        $this->assertNotNull($result->items[0]);
        $this->assertNotNull($result->items[0]->id);
        $this->assertNotNull($result->items[0]->scores);
        $this->assertEqual(count($result->items[0]->scores), 3);
        $this->assertFalse(isset($result->items[0]->errors));
        $this->assertEqual($result->items[0]->grademax, 90);
        $this->assertNotNull($result->items[0]->scores[0]);
        $this->assertEqual($result->items[0]->scores[0]->rawgrade, 85);
        $this->assertNotNull($result->items[0]->scores[1]);
        $this->assertEqual($result->items[0]->scores[1]->rawgrade, 50);
        $this->assertNotNull($result->items[0]->scores[2]);
        $this->assertEqual($result->items[0]->scores[2]->rawgrade, 0);
        /*
        echo "<pre>";
        var_export($result->items[0]);
        echo "</pre>";
        */
        $xml = iclicker_service::encode_gradebook_results($result);
        $this->assertNull($xml); // No errors.

        // Test saving multiple items at once.
        $gradebook->course_id = $this->courseid;
        $gradebook->items = array();

        $gradeitem1 = new stdClass();
        $gradeitem1->name = $testitemname2;
        $gradeitem1->points_possible = 100;
        $gradeitem1->type = null; // Default.
        $gradeitem1->scores = array();
        $gradebook->items[] = $gradeitem1;

        $gradeitem2 = new stdClass();
        $gradeitem2->name = $testitemname3;
        $gradeitem2->points_possible = 50;
        $gradeitem2->type = 'stuff';
        $gradeitem2->scores = array();
        $gradebook->items[] = $gradeitem2;

        $score = new stdClass();
        $score->userid = 1;
        $score->score = 80.0;
        $gradeitem1->scores[] = $score;

        $score = new stdClass();
        $score->userid = 2;
        $score->score = 90.0;
        $gradeitem1->scores[] = $score;

        $score = new stdClass();
        $score->userid = 2;
        $score->score = 45.0;
        $gradeitem2->scores[] = $score;

        $score = new stdClass();
        $score->userid = 3;
        $score->score = 40.0;
        $gradeitem2->scores[] = $score;

        $result = iclicker_service::save_gradebook($gradebook);
        $this->assertNotNull($result);
        $this->assertNotNull($result->course_id);
        $this->assertNotNull($result->items);
        $this->assertNotNull($result->default_category_id);
        $this->assertEqual($result->course_id, $this->courseid);
        $this->assertEqual(count($result->items), 2);
        $this->assertNotNull($result->items[0]);
        $this->assertNotNull($result->items[0]->id);
        $this->assertNotNull($result->items[0]->scores);
        $this->assertEqual(count($result->items[0]->scores), 2);
        $this->assertFalse(isset($result->items[0]->errors));
        $this->assertEqual($result->items[0]->grademax, 100);
        $this->assertNotNull($result->items[0]->scores[0]);
        $this->assertEqual($result->items[0]->scores[0]->rawgrade, 80);
        $this->assertNotNull($result->items[0]->scores[1]);
        $this->assertEqual($result->items[0]->scores[1]->rawgrade, 90);
        $this->assertNotNull($result->items[1]);
        $this->assertNotNull($result->items[1]->id);
        $this->assertNotNull($result->items[1]->scores);
        $this->assertEqual(count($result->items[1]->scores), 2);
        $this->assertFalse(isset($result->items[1]->errors));
        $this->assertEqual($result->items[1]->grademax, 50);
        $this->assertNotNull($result->items[1]->scores[0]);
        $this->assertEqual($result->items[1]->scores[0]->rawgrade, 45);
        $this->assertNotNull($result->items[1]->scores[1]);
        $this->assertEqual($result->items[1]->scores[1]->rawgrade, 40);

        $xml = iclicker_service::encode_gradebook_results($result);
        $this->assertNull($xml);

    }

    // TODO - disabled
    public function disabled_test_save_ws() {
        $result = iclicker_service::ws_get_students();
        $this->assertNotNull($result);

        $clickerreg = new stdClass();
        $clickerreg->ownerid = '101'; // student01
        $clickerreg->clickerid = '11111111';
        // NOTE: disable this to avoid putting test data into the real domain of systems.
        /*
                $result = iclicker_service::ws_save_clicker($clickerreg);
                $this->assertNotNull($result);
        */
        $result = iclicker_service::ws_get_student('student01');
        $this->assertNotNull($result);
    }

}
